!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module hmix_del2

!***********************************************************************
!
!     this module contains routine for computing Laplacian horizontal
!     diffusion of momentum and tracers
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use operators
      use topostress
      use diagnostics
      use grid
      use advection

      implicit none

!***********************************************************************

      contains

!***********************************************************************

      subroutine hdiffu_del2(k,HDUK,HDVK,am,lvariable_hmix,AMF)

!-----------------------------------------------------------------------
!
!     horizontial diffusion of momentum
!
!     HDiff(Ub) = am*Del**2(Ub)  (laplacian mixing)
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k  ! depth level index

      real (kind=dbl_kind), intent(in) ::
     &  am                  ! horizontal momentum mixing coefficient

      logical (kind=log_kind), intent(in) ::
     &  lvariable_hmix      ! flag for spatial variation of mix coeffs

      real (kind=dbl_kind), dimension(:,:), intent(in) ::
     &  AMF                 ! spatial variation of mixing coeff

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) :: 
     &  HDUK,               ! Hdiff(Ub) at level k
     &  HDVK                ! Hdiff(Vb) at level k

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  UTMP, VTMP,         ! modified velocities to use with topostress
     &  HDIFFCFL            ! for cfl number diagnostics
      
!-----------------------------------------------------------------------
!
!     laplacian mixing
!
!-----------------------------------------------------------------------

      if (ltopostress) then
        UTMP = merge(UVEL(:,:,k,mixtime) - TSU,
     &               UVEL(:,:,k,mixtime), k <= KMU)
        VTMP = merge(VVEL(:,:,k,mixtime) - TSV,
     &               VVEL(:,:,k,mixtime), k <= KMU)
        call del2u(k,HDUK,HDVK,UTMP,VTMP)
      else
        call del2u(k,HDUK,HDVK,UVEL(:,:,k,mixtime),
     &                         VVEL(:,:,k,mixtime))
      endif

!-----------------------------------------------------------------------
!
!     multiply by viscosity
!
!-----------------------------------------------------------------------

      HDUK = am*HDUK
      HDVK = am*HDVK

!-----------------------------------------------------------------------
!
!     compute horiz diffusion cfl diagnostics if required
!
!-----------------------------------------------------------------------

      if (ldiag_cfl) then
 
        if (lvariable_hmix) then
          HDIFFCFL = merge(c4*am*AMF*(DXUR**2 + DYUR**2), c0, KMU > k)
        else
          HDIFFCFL = merge(c4*am*    (DXUR**2 + DYUR**2), c0, KMU > k)
        endif
        HDIFFCFL = abs(HDIFFCFL)
        call cfl_hdiff(k,HDIFFCFL,2)
            
      endif

!-----------------------------------------------------------------------

      end subroutine hdiffu_del2

!***********************************************************************

      subroutine hdifft_del2(k,HDTK,ah,lvariable_hmix,AHF)

!-----------------------------------------------------------------------
!
!     horizontial diffusion of tracers
!
!     HDiff(Tb) = ah*Del**2(Tb)  (laplacian mixing)
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k  ! depth level index

      real (kind=dbl_kind), intent(in) ::
     &  ah                  ! horizontal tracer mixing coefficient

      logical (kind=log_kind), intent(in) ::
     &  lvariable_hmix      ! flag for spatial variation of mix coeffs

      real (kind=dbl_kind), dimension(:,:), intent(in) ::
     &  AHF                 ! spatial variation of mixing coeff

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,nt) :: 
     &  HDTK                ! HDIFF(T) for tracer n at level k

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  HDIFFCFL            ! for cfl number diagnostics

!-----------------------------------------------------------------------
!
!     laplacian mixing
!
!-----------------------------------------------------------------------

      call del2t(k,HDTK)

!-----------------------------------------------------------------------
!
!     multiply by diffusivity
!
!-----------------------------------------------------------------------

      HDTK = ah*HDTK

!-----------------------------------------------------------------------
!
!     compute horiz diffusion cfl diagnostics if required
!
!-----------------------------------------------------------------------

      if (ldiag_cfl) then
 
        if (lvariable_hmix) then
          HDIFFCFL = merge(c4*ah*AHF*(DXTR**2 + DYTR**2), c0, KMT > k)
        else
          HDIFFCFL = merge(c4*ah*    (DXTR**2 + DYTR**2), c0, KMT > k)
        endif
        HDIFFCFL = abs(HDIFFCFL)
        call cfl_hdiff(k,HDIFFCFL,1)
            
      endif

!-----------------------------------------------------------------------

      end subroutine hdifft_del2

!***********************************************************************

      subroutine hdifft_mpdcd_del2(k,HDTK,ah,lvariable_hmix,AHF)

!-----------------------------------------------------------------------
!
!     horizontial diffusion of tracers
!
!     HDiff(Tb) = ah*Del**2(Tb)  (laplacian mixing)
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k  ! depth level index

      real (kind=dbl_kind), intent(in) ::
     &  ah                  ! horizontal tracer mixing coefficient

      logical (kind=log_kind), intent(in) ::
     &  lvariable_hmix      ! flag for spatial variation of mix coeffs

      real (kind=dbl_kind), dimension(:,:), intent(in) ::
     &  AHF                 ! spatial variation of mixing coeff

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,nt) :: 
     &  HDTK                ! HDIFF(T) for tracer n at level k

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  HDIFFCFL            ! for cfl number diagnostics

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &   CC,CN,CS,CE,CW        ! coeff of 5pt stencil for Del**2

!-----------------------------------------------------------------------
!
!     implement boundary conditions by setting
!     stencil coefficients to zero at land points.
!
!-----------------------------------------------------------------------

      CN = merge(DTN, c0, (k.le.KMTN).and.(k.le.KMT))
      CS = merge(DTS, c0, (k.le.KMTS).and.(k.le.KMT))
      CE = merge(DTE, c0, (k.le.KMTE).and.(k.le.KMT))
      CW = merge(DTW, c0, (k.le.KMTW).and.(k.le.KMT))

      CC = -(CN + CS + CE + CW)  ! central coefficient

!-----------------------------------------------------------------------
!
!     laplacian mixing
!
!-----------------------------------------------------------------------

      call del2t_mpdcd(k,HDTK,CC,CN,CS,CE,CW)

!-----------------------------------------------------------------------
!
!     calculate diffusive fluxes for n > 2, then zero out HDTK
!
!-----------------------------------------------------------------------

      do n = 3,nt
        
        if (k == 1) then
          call n_eoshift(WORK,TRACER(:,:,k,n,mixtime))
          FRVNH(:,:,2,n) = -ah*CN*(WORK-TRACER(:,:,k,n,mixtime))
          call s_eoshift(WORK,TRACER(:,:,k,n,mixtime))
          FRVSH(:,:,2,n) = -ah*CS*(WORK-TRACER(:,:,k,n,mixtime))
          call e_eoshift(WORK,TRACER(:,:,k,n,mixtime))
          FRVEH(:,:,2,n) = -ah*CE*(WORK-TRACER(:,:,k,n,mixtime))
          call w_eoshift(WORK,TRACER(:,:,k,n,mixtime))
          FRVWH(:,:,2,n) = -ah*CW*(WORK-TRACER(:,:,k,n,mixtime))
        else
          call n_eoshift(WORK,TRACER(:,:,k,n,mixtime))
          FRVNH(:,:,3,n) = -ah*CN*(WORK-TRACER(:,:,k,n,mixtime))
          call s_eoshift(WORK,TRACER(:,:,k,n,mixtime))
          FRVSH(:,:,3,n) = -ah*CS*(WORK-TRACER(:,:,k,n,mixtime))
          call e_eoshift(WORK,TRACER(:,:,k,n,mixtime))
          FRVEH(:,:,3,n) = -ah*CE*(WORK-TRACER(:,:,k,n,mixtime))
          call w_eoshift(WORK,TRACER(:,:,k,n,mixtime))
          FRVWH(:,:,3,n) = -ah*CW*(WORK-TRACER(:,:,k,n,mixtime))
        endif

        HDTK(:,:,n) = c0

      enddo

!-----------------------------------------------------------------------
!
!     multiply by diffusivity for n = 1:2
!
!-----------------------------------------------------------------------

      HDTK(:,:,1:2) = ah*HDTK(:,:,1:2)

!-----------------------------------------------------------------------
!
!     compute horiz diffusion cfl diagnostics if required
!
!-----------------------------------------------------------------------

      if (ldiag_cfl) then
 
        if (lvariable_hmix) then
          HDIFFCFL = merge(c4*ah*AHF*(DXTR**2 + DYTR**2), c0, KMT > k)
        else
          HDIFFCFL = merge(c4*ah*    (DXTR**2 + DYTR**2), c0, KMT > k)
        endif
        HDIFFCFL = abs(HDIFFCFL)
        call cfl_hdiff(k,HDIFFCFL,1)
            
      endif

!-----------------------------------------------------------------------

      end subroutine hdifft_mpdcd_del2

!***********************************************************************

      end module hmix_del2

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
