!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module hmix_del4

!***********************************************************************
!
!     this module contains routine for computing biharmonic horizontal
!     diffusion of momentum and tracers
!
!-----------------------------------------------------------------------

      use kinds_mod
      use model_size
      use domain
      use constants
      use operators
      use grid
      use diagnostics
      use advection

      implicit none

!***********************************************************************

      contains

!***********************************************************************

      subroutine hdiffu_del4(k,HDUK,HDVK,am,lvariable_hmix,AMF)

!-----------------------------------------------------------------------
!
!     horizontial diffusion of momentum
!
!     HDiff(Ub) = am*Del**4(Ub)  (biharmonic mixing)
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k  ! depth level index

      real (kind=dbl_kind), intent(in) ::
     &  am                  ! horizontal momentum mixing coefficient

      logical (kind=log_kind), intent(in) ::
     &  lvariable_hmix      ! flag for spatial variation of mix coeffs

      real (kind=dbl_kind), dimension(:,:), intent(in) ::
     &  AMF                 ! spatial variation of mixing coeff

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) :: 
     &  HDUK,               ! Hdiff(Ub) at level k
     &  HDVK                ! Hdiff(Vb) at level k

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  HDIFFCFL            ! local hdiff cfl number for diagnostics

!-----------------------------------------------------------------------
!
!     biharmonic mixing 
!
!-----------------------------------------------------------------------

      call del4u(k,HDUK,HDVK,lvariable_hmix,AMF)

!-----------------------------------------------------------------------
!
!     multiply by viscosity
!
!-----------------------------------------------------------------------

      HDUK = am*HDUK
      HDVK = am*HDVK

!-----------------------------------------------------------------------
!
!     compute cfl diagnostics if it is time
!
!-----------------------------------------------------------------------

      if (ldiag_cfl) then
        if (lvariable_hmix) then
            HDIFFCFL = merge(-c16*am*AMF*(DXUR**2 + DYUR**2)**2,
     &                       c0, KMU > k)
        else
            HDIFFCFL = merge(-c16*am*(DXUR**2 + DYUR**2)**2, 
     &                       c0, KMU > k)
        endif
        HDIFFCFL = abs(HDIFFCFL)
        call cfl_hdiff(k,HDIFFCFL,2)
      endif

!-----------------------------------------------------------------------

      end subroutine hdiffu_del4

!***********************************************************************

      subroutine hdifft_del4(k,HDTK,ah,lvariable_hmix,AHF)

!-----------------------------------------------------------------------
!
!     horizontial diffusion of tracers
!
!     HDiff(Tb) = ah*Del**4(Tb)  (biharmonic mixing)
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k  ! depth level index

      real (kind=dbl_kind), intent(in) ::
     &  ah                  ! horizontal tracer mixing coefficient

      logical (kind=log_kind), intent(in) ::
     &  lvariable_hmix      ! flag for spatial variation of mix coeffs

      real (kind=dbl_kind), dimension(:,:), intent(in) ::
     &  AHF                 ! spatial variation of mixing coeff

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,nt) :: 
     &  HDTK                ! HDIFF(T) for tracer n at level k

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  HDIFFCFL            ! local hdiff cfl number for diagnostics

!-----------------------------------------------------------------------
!
!     biharmonic mixing 
!
!-----------------------------------------------------------------------

      call del4t(k,HDTK,lvariable_hmix,AHF)

!-----------------------------------------------------------------------
!
!     multiply by diffusivity
!
!-----------------------------------------------------------------------

      HDTK = ah*HDTK

!-----------------------------------------------------------------------
!
!     compute cfl diagnostics if it is time
!
!-----------------------------------------------------------------------

      if (ldiag_cfl) then
        if (lvariable_hmix) then
            HDIFFCFL = merge(-c16*ah*AHF*(DXTR**2 + DYTR**2)**2,
     &                       c0, KMT > k)
        else
            HDIFFCFL = merge(-c16*ah*(DXTR**2 + DYTR**2)**2, 
     &                       c0, KMT > k)
        endif
        HDIFFCFL = abs(HDIFFCFL)
        call cfl_hdiff(k,HDIFFCFL,1)
      endif

!-----------------------------------------------------------------------

      end subroutine hdifft_del4

!***********************************************************************

      subroutine hdifft_mpdcd_del4(k,HDTK,ah,lvariable_hmix,AHF)

!-----------------------------------------------------------------------
!
!     horizontial diffusion of tracers
!
!     HDiff(Tb) = ah*Del**4(Tb)  (biharmonic mixing)
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k  ! depth level index

      real (kind=dbl_kind), intent(in) ::
     &  ah                  ! horizontal tracer mixing coefficient

      logical (kind=log_kind), intent(in) ::
     &  lvariable_hmix      ! flag for spatial variation of mix coeffs

      real (kind=dbl_kind), dimension(:,:), intent(in) ::
     &  AHF                 ! spatial variation of mixing coeff

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,nt) :: 
     &  HDTK                ! HDIFF(T) for tracer n at level k

!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt) :: 
     &  HDIFFCFL            ! local hdiff cfl number for diagnostics

      real (kind=dbl_kind), dimension(imt,jmt) ::
     &   CC,CN,CS,CE,CW        ! coeff of 5pt stencil for Del**2

!-----------------------------------------------------------------------
!
!     implement boundary conditions by setting
!     stencil coefficients to zero at land points.
!
!-----------------------------------------------------------------------

      CN = merge(DTN, c0, (k.le.KMTN).and.(k.le.KMT))
      CS = merge(DTS, c0, (k.le.KMTS).and.(k.le.KMT))
      CE = merge(DTE, c0, (k.le.KMTE).and.(k.le.KMT))
      CW = merge(DTW, c0, (k.le.KMTW).and.(k.le.KMT))

      CC = -(CN + CS + CE + CW)  ! central coefficient

!-----------------------------------------------------------------------
!
!     biharmonic mixing 
!
!-----------------------------------------------------------------------

      call del4t_mpdcd(k,HDTK,lvariable_hmix,AHF,CC,CN,CS,CE,CW)

!-----------------------------------------------------------------------
!
!     calculate diffusive fluxes for n > 2, then zero out HDTK.
!     NOTE that for n > 2 HDTK holds Del2(T), instead of Del4(T).
!
!-----------------------------------------------------------------------

      do n = 3,nt

        if (k == 1) then
          call n_eoshift(WORK,HDTK(:,:,n))
          FRVNH(:,:,2,n) = -ah*CN*(WORK-HDTK(:,:,n))
          call s_eoshift(WORK,HDTK(:,:,n))
          FRVSH(:,:,2,n) = -ah*CS*(WORK-HDTK(:,:,n))
          call e_eoshift(WORK,HDTK(:,:,n))
          FRVEH(:,:,2,n) = -ah*CE*(WORK-HDTK(:,:,n))
          call w_eoshift(WORK,HDTK(:,:,n))
          FRVWH(:,:,2,n) = -ah*CW*(WORK-HDTK(:,:,n))
        else
          call n_eoshift(WORK,HDTK(:,:,n))
          FRVNH(:,:,3,n) = -ah*CN*(WORK-HDTK(:,:,n))
          call s_eoshift(WORK,HDTK(:,:,n))
          FRVSH(:,:,3,n) = -ah*CS*(WORK-HDTK(:,:,n))
          call e_eoshift(WORK,HDTK(:,:,n))
          FRVEH(:,:,3,n) = -ah*CE*(WORK-HDTK(:,:,n))
          call w_eoshift(WORK,HDTK(:,:,n))
          FRVWH(:,:,3,n) = -ah*CW*(WORK-HDTK(:,:,n))
        endif

        HDTK(:,:,n) = c0

      enddo

!-----------------------------------------------------------------------
!
!     multiply by diffusivity for n = 1:2
!
!-----------------------------------------------------------------------

      HDTK(:,:,1:2) = ah*HDTK(:,:,1:2)

!-----------------------------------------------------------------------
!
!     compute cfl diagnostics if it is time
!
!-----------------------------------------------------------------------

      if (ldiag_cfl) then
        if (lvariable_hmix) then
            HDIFFCFL = merge(-c16*ah*AHF*(DXTR**2 + DYTR**2)**2,
     &                       c0, KMT > k)
        else
            HDIFFCFL = merge(-c16*ah*(DXTR**2 + DYTR**2)**2, 
     &                       c0, KMT > k)
        endif
        HDIFFCFL = abs(HDIFFCFL)
        call cfl_hdiff(k,HDIFFCFL,1)
      endif

!-----------------------------------------------------------------------

      end subroutine hdifft_mpdcd_del4

!***********************************************************************

      end module hmix_del4

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
