!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

      module horizontal_mix

!***********************************************************************

      use kinds_mod
      use model_size
      use domain
      use constants
      use io
      use operators
      use global_reductions
      use hmix_del2
      use hmix_del4
      use hmix_smag
      use hmix_gm
      use exit_mod

      implicit none
      save

!-----------------------------------------------------------------------
!
!     horizontal mixing variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), parameter :: ! choices for mixing type
     &  hmix_momentum_type_del2 = 1,
     &  hmix_momentum_type_del4 = 2,
     &  hmix_momentum_type_smag = 3,
     &  hmix_tracer_type_del2 = 1,
     &  hmix_tracer_type_del4 = 2,
     &  hmix_tracer_type_smag = 3,
     &  hmix_tracer_type_gm   = 4,
     &  hmix_tracer_type_mpdcd_del2 = 5,
     &  hmix_tracer_type_mpdcd_del4 = 6,
     &  hmix_tracer_type_mpdcd_gm   = 7

      integer (kind=int_kind) ::  ! integer choice for type of mixing
     &  hmix_momentum_itype,
     &  hmix_tracer_itype

      real (kind=dbl_kind) ::
     &  ah,              ! horizontal tracer   mixing coefficient
     &  am               ! horizontal momentum mixing coefficient

      logical (kind=log_kind) ::
     &  lauto_hmix,      ! automatically computing mixing coeffs
     &  lvariable_hmix   ! spatially varying mixing coeffs

      real (kind=dbl_kind), dimension(:,:), allocatable :: 
     &  AHF,             ! variable mixing factor for tracer   mixing
     &  AMF              ! variable mixing factor for momentum mixing

      real (kind=dbl_kind), dimension(:,:,:), allocatable :: 
     &  HDTKP            ! needed for MPDCD advection

!***********************************************************************

      contains

!***********************************************************************

      subroutine init_horizontal_mix

!-----------------------------------------------------------------------
!
!     initialize various mixing parameters based on namelist input
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     local variables
!
!-----------------------------------------------------------------------

      integer (kind=int_kind) :: nml_error  ! error flag for namelist

      character (char_len) ::  ! character choice for type of mixing
     &  hmix_momentum_choice,
     &  hmix_tracer_choice

      namelist /hmix_nml/ hmix_momentum_choice, hmix_tracer_choice,
     &                    lauto_hmix, lvariable_hmix, ah, am

      real (kind=dbl_kind) ::
     &  ahfmin, ahfmax, amfmin, amfmax

      character (char_len) ::
     &  tadvect_ctype      ! character string for tracer advect choice

      namelist /advect_nml/ tadvect_ctype

!-----------------------------------------------------------------------
!
!     read namelist input
!
!-----------------------------------------------------------------------

      hmix_momentum_choice = 'unknown_hmix_momentum_choice'
      hmix_tracer_choice = 'unknown_hmix_tracer_choice'
      lauto_hmix = .false.
      lvariable_hmix = .false.
      ah = c0
      am = c0

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   10   continue  !*** keep reading until find right namelist
        read(nml_in, nml=hmix_nml,err=10,end=20)
        close(nml_in)
        nml_error = 0
   20   continue
      endif

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP('ERROR reading hmix_nml')
      endif

      if (my_task == master_task) then
        write(stdout,1100)
        write(stdout,*)
        write(stdout,*) 'Horizontal mixing options'
        write(stdout,*)
        write(stdout,1100)
 1100   format(72('-')) 

        select case (hmix_momentum_choice(1:4))
        case ('del2')
          hmix_momentum_itype = hmix_momentum_type_del2
          write(stdout,*) 'Laplacian horizontal momentum mixing used.'
        case ('del4')
          hmix_momentum_itype = hmix_momentum_type_del4
          write(stdout,*) 'Biharmonic horizontal momentum mixing used.'
        case ('smag')
          hmix_momentum_itype = hmix_momentum_type_smag
          write(stdout,*) 'Smagorinsky horizontal momentum mixing used.'
        case ('gent')
          hmix_momentum_itype = -1000
        case default
          hmix_momentum_itype = -2000
        end select

        select case (hmix_tracer_choice(1:4))
        case ('del2')
          hmix_tracer_itype = hmix_tracer_type_del2
          write(stdout,*) 'Laplacian horizontal tracer   mixing chosen.'
        case ('del4')
          hmix_tracer_itype = hmix_tracer_type_del4
          write(stdout,*) 'Biharmonic horizontal tracer   mixing used.'
        case ('smag')
          hmix_tracer_itype = hmix_tracer_type_smag
          write(stdout,*) 'Smagorinsky horizontal tracer   mixing used.'
        case ('gent')
          hmix_tracer_itype = hmix_tracer_type_gm
          write(stdout,*) 'Gent-McWilliams tracer mixing used.'
        case default
          hmix_tracer_itype = -1000
        end select

        if (.not. lauto_hmix) then
          write(stdout,*)
     &         'Using input horizontal mixing coefficients:'
          write(stdout,1000) '  ah =',ah
          write(stdout,1000) '  am =',am
 1000     format(a7,2x,1pe12.5)
        endif

        if (.not. lvariable_hmix) then
          write(stdout,*) 'Variable horizontal mixing disabled'
        endif

      endif

!-----------------------------------------------------------------------
!
!     read namelist input from advection to check for MPDCD
!
!-----------------------------------------------------------------------

      tadvect_ctype = 'standard'

      if (my_task == master_task) then
        nml_error = -1
        open (nml_in, file=nml_filename, status='old')
   11   continue  !*** keep reading until find right namelist
        read(nml_in, nml=advect_nml,err=11,end=21)
        close(nml_in)
        nml_error = 0
   21   continue
      end if

      call broadcast_scalar(nml_error, master_task)
      if (nml_error /= 0) then
        call exit_POP
     &    ('ERROR reading advection namelist (in init_horizontal_mix)')
      endif

      if (trim(tadvect_ctype) == 'mpdcd') then
        write(stdout,*) ' '
        write(stdout,*) 'MPDCD advection will modify Diffusive terms.'
        select case (hmix_tracer_choice(1:4))
        case ('del2')
          hmix_tracer_itype = hmix_tracer_type_mpdcd_del2
        case ('del4')
          hmix_tracer_itype = hmix_tracer_type_mpdcd_del4
        case ('gent')
          hmix_tracer_itype = hmix_tracer_type_mpdcd_gm
        case default
          hmix_tracer_itype = -1000
        end select
      endif

      call broadcast_scalar(lauto_hmix,          master_task)
      call broadcast_scalar(lvariable_hmix,      master_task)
      call broadcast_scalar(hmix_momentum_itype, master_task)
      call broadcast_scalar(hmix_tracer_itype,   master_task)
      call broadcast_scalar(ah,                  master_task)
      call broadcast_scalar(am,                  master_task)

      if (hmix_momentum_itype == -1000) then
        call exit_POP(
     &          'Gent-McWilliams can only be used for tracer mixing')
      else if (hmix_momentum_itype == -2000) then
        call exit_POP( 'Unknown type for horizontal momentum mixing')
      endif

      if (hmix_tracer_itype == -1000) then
        call exit_POP( 'Unknown type for horizontal tracer mixing')
      endif

      !***
      !*** check compatibility of some of these choices
      !***

      if ((hmix_momentum_itype == hmix_momentum_type_smag .and.
     &     hmix_tracer_itype   /= hmix_tracer_type_smag) .or. 
     &    (hmix_momentum_itype /= hmix_momentum_type_smag .and.
     &     hmix_tracer_itype   == hmix_tracer_type_smag)) then
        call exit_POP(
     &    'init_hmix: Both horizontal mixing types must be Smagorinsky')
      endif

!-----------------------------------------------------------------------
!
!     automatically set horizontal mixing coefficients if requested 
!
!-----------------------------------------------------------------------

      if (lauto_hmix) then

        select case (hmix_momentum_itype)
        case (hmix_momentum_type_del2)
          ! scale to 1e7 at 1/2 degree
          am = 1.0e7_dbl_kind*(720.0_dbl_kind/float(imt_global))
        case (hmix_momentum_type_del4)
          am = -0.6e20_dbl_kind*(1280.0_dbl_kind/float(imt_global))
        end select

        select case (hmix_tracer_itype)
        case (hmix_tracer_type_del2)
          ! scale to 1e7 at 1/2 degree
          ah = 1.0e7_dbl_kind*(720.0_dbl_kind/float(imt_global))
        case (hmix_tracer_type_del4)
          ah = -0.2e20_dbl_kind*(1280.0_dbl_kind/float(imt_global))
        case (hmix_tracer_type_mpdcd_del2)
          ah = 1.0e7_dbl_kind*(720.0_dbl_kind/float(imt_global))
        case (hmix_tracer_type_mpdcd_del4)
          ah = -0.2e20_dbl_kind*(1280.0_dbl_kind/float(imt_global))
        end select

        if (my_task == master_task) then
          write(stdout,*)
     &         'Horizontal mixing coefficients computed automatically:'
          write(stdout,1000) '  ah =',ah
          write(stdout,1000) '  am =',am
        endif
      endif

!-----------------------------------------------------------------------
!
!     set variable mixing arrays if requested
!
!     functions describing spatial dependence of horizontal mixing
!     coefficients:   ah -> ah*AHF and am -> am*AHM  
!     for laplacian  mixing they scale like (cell area)**0.5
!     for biharmonic mixing they scale like (cell area)**1.5
!     note: these forms assume a global mesh with a grid line along
!     the equator, and the mixing coefficients are the set relative
!     to the average grid spacing at the equator - for cells of
!     this size AMF = AHF = 1.0
!
!-----------------------------------------------------------------------

      if (lvariable_hmix) then
        allocate(AMF(imt,jmt), AHF(imt,jmt))

        select case (hmix_momentum_itype)
        case (hmix_momentum_type_del2)
!RSncom:  set variable viscocity for x3-prime run
c         AMF = sqrt(UAREA/(c2*pi*radius/imt_global)**2)
          AMF = c4*am*(DXUR**2+DYUR**2)*dtu
          where(AMF.gt.p5)
            AMF = p5/AMF
          elsewhere
            AMF = c1
          endwhere
!RSncom
        case (hmix_momentum_type_del4)
          AMF = (UAREA/uarea_equator)**1.5
        case default
          AMF = c1
        end select
        amfmin = global_minval(AMF)
        amfmax = global_maxval(AMF)

        select case (hmix_tracer_itype)
        case (hmix_tracer_type_del2)
          AHF = sqrt(TAREA/(c2*pi*radius/imt_global)**2)
        case (hmix_tracer_type_del4)
          AHF = (TAREA/uarea_equator)**1.5
        case (hmix_tracer_type_mpdcd_del2)
          AHF = sqrt(TAREA/(c2*pi*radius/imt_global)**2)
        case (hmix_tracer_type_mpdcd_del4)
          AHF = (TAREA/uarea_equator)**1.5
        case default
          AHF = c1
        end select
        ahfmin = global_minval(AHF)
        ahfmax = global_maxval(AHF)

        if (my_task == master_task) then
          write(stdout,*) 'Variable horizontal mixing enabled'
          write(stdout,1010) ahfmin, ahfmax
          write(stdout,1020) amfmin, amfmax
 1010     format('  Min AHF =',1x,1pe12.5,3x,'Max AHF =',1x,1pe12.5)
 1020     format('  Min AMF =',1x,1pe12.5,3x,'Max AMF =',1x,1pe12.5)
        endif
      else
        allocate(AMF(1,1), AHF(1,1))
      endif
 
!-----------------------------------------------------------------------
!
!     calculate additional coefficients if necessary
!
!-----------------------------------------------------------------------
 
      if (hmix_momentum_itype == hmix_momentum_type_smag) then
        call init_smag  ! coefficients used by Smagorinsky viscosity
      endif

!claagmp:--
      if (hmix_tracer_itype == hmix_tracer_type_gm .or.
     &    hmix_tracer_itype == hmix_tracer_type_mpdcd_gm) then
        call init_gm  ! variables used by GM parameterization
      endif

!-----------------------------------------------------------------------
!
!     allocate additional array if MPDCD advection is selected
!
!-----------------------------------------------------------------------

      if (hmix_tracer_itype == hmix_tracer_type_mpdcd_del2 .or.
     &    hmix_tracer_itype == hmix_tracer_type_mpdcd_del4 .or.
     &    hmix_tracer_itype == hmix_tracer_type_mpdcd_gm) then
        allocate( HDTKP(imt,jmt,nt) )
        HDTKP = c0
      endif
!claa--
!-----------------------------------------------------------------------

      end subroutine init_horizontal_mix

!***********************************************************************

      subroutine hdiffu(k,HDUK,HDVK)

!-----------------------------------------------------------------------
!
!     horizontial diffusion of momentum
!
!     this is only a wrapper which calls a specific routine for the
!     type of horizontal mixing chosen in init_horizontal_mix
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k   ! depth level index

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt), intent(out) :: 
     &  HDUK,               ! returned as Hdiff(U) at level k
     &  HDVK                ! returned as Hdiff(V) at level k

!-----------------------------------------------------------------------

      select case (hmix_momentum_itype)

      case (hmix_momentum_type_del2)
        call hdiffu_del2(k,HDUK,HDVK,am,lvariable_hmix,AMF)
      case (hmix_momentum_type_del4)
        call hdiffu_del4(k,HDUK,HDVK,am,lvariable_hmix,AMF)
      case (hmix_momentum_type_smag)
        call hdiffu_smag(k,HDUK,HDVK)
      end select

!-----------------------------------------------------------------------

      end subroutine hdiffu

!***********************************************************************

      subroutine hdifft(k,HDTK)

!-----------------------------------------------------------------------
!
!     horizontial diffusion of tracers
!
!     this is only a wrapper which calls a specific routine for the
!     type of horizontal mixing chosen in init_horizontal_mix
!
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
!
!     input variables:
!
!-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: k   ! depth level index

!-----------------------------------------------------------------------
!
!     output variables:
!
!-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(imt,jmt,nt), intent(out) :: 
     &  HDTK                ! Hdiff(T) for nth tracer at level k

!-----------------------------------------------------------------------

      select case (hmix_tracer_itype)
      case (hmix_tracer_type_del2)
        call hdifft_del2(k,HDTK,ah,lvariable_hmix,AHF)
      case (hmix_tracer_type_del4)
        call hdifft_del4(k,HDTK,ah,lvariable_hmix,AHF)
      case (hmix_tracer_type_smag)
        call hdifft_smag(k,HDTK)
      case (hmix_tracer_type_gm)
        call hdifft_gm(k,HDTK,ah)
      case (hmix_tracer_type_mpdcd_del2)
        if ( k == 1 ) then
          call hdifft_mpdcd_del2(k  ,HDTK ,ah,lvariable_hmix,AHF)
          call hdifft_mpdcd_del2(k+1,HDTKP,ah,lvariable_hmix,AHF)
        elseif ( k == km ) then
          HDTK = HDTKP
        else
          HDTK = HDTKP
          call hdifft_mpdcd_del2(k+1,HDTKP,ah,lvariable_hmix,AHF)
        endif
      case (hmix_tracer_type_mpdcd_del4)
        if ( k == 1 ) then
          call hdifft_mpdcd_del4(k  ,HDTK ,ah,lvariable_hmix,AHF)
          call hdifft_mpdcd_del4(k+1,HDTKP,ah,lvariable_hmix,AHF)
        elseif ( k == km ) then
          HDTK = HDTKP
        else
          HDTK = HDTKP
          call hdifft_mpdcd_del4(k+1,HDTKP,ah,lvariable_hmix,AHF)
        endif
      case (hmix_tracer_type_mpdcd_gm)
!claagmp:--
        if ( k == 1 ) then
          call hdifft_gm(k  ,HDTK ,ah)
          call hdifft_gm(k+1,HDTKP,ah)
        elseif ( k == km ) then
          HDTK = HDTKP
        else
          HDTK = HDTKP
          call hdifft_gm(k+1,HDTKP,ah)
        endif
!claa--
      end select

!-----------------------------------------------------------------------
!     do time average of horizontal mixing if required.
!-----------------------------------------------------------------------

      if (ltavg_on .and. mix_pass /= 1 .and. 
     &    hmix_tracer_itype /= hmix_tracer_type_mpdcd_del2 .and. 
     &    hmix_tracer_itype /= hmix_tracer_type_mpdcd_del4) then
        call tavg_hdifft(k,HDTK)
      endif

!-----------------------------------------------------------------------

      end subroutine hdifft

!***********************************************************************

      end module horizontal_mix

!|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
